/* 
    I2CDriver.h - I2C 100-400KHz single master driver.  Definitions for supporting
    data structures to COGC driver.
*/

#ifndef __I2C_DRIVER_H__
#define __I2C_DRIVER_H__

#include <propeller.h>

/* I2C command types */
typedef enum I2C_CMD
{
    I2C_CMD_IDLE,           /* Bus quiet and ready for new command */
    I2C_CMD_INIT,           /* Initialize the bus */
    I2C_CMD_LOCKED,         /* Marks the bus as locked but not yet active */          
    I2C_CMD_SEND,           /* Send data bytes to the bus at a register address */
    I2C_CMD_RECEIVE         /* Recieve data bytes from the bus at a register address */
} I2C_CMD;


/* I2C status return types */
typedef enum I2C_RESULT
{
    I2C_OK = 0,
    I2C_ERR_UNKNOWN_CMD,
    I2C_ERR_SEND_HDR,
    I2C_ERR_SEND,
    I2C_ERR_RECEIVE_HDR,
    I2C_ERR_RECEIVE
} I2C_RESULT;
 

/* Command mailbox structure - Groups parameters used to pass commands
   to the cog controlling the I2C bus.*/
typedef struct I2C_MAILBOX
{
    volatile uint32_t cmd;       /* Primative I2C command (SEND,RECV, ect) */
    volatile uint32_t sts;       /* Last command status */
    volatile uint8_t  hdr;       /* I2C address header  */
    volatile uint16_t reg;       /* Register address for register read/write */
    volatile uint8_t  reg_count; /* Number of register bytes to send 1 or 2*/    
    volatile uint8_t  count;     /* Number of bytes to be sent/recv'd */
    uint8_t *buffer;             /* Pointer to data to be sent/recv'd */
} I2C_MAILBOX;


/* Initialization structure - Groups parameters used to to start the
   I2C cog */
typedef struct 
{
    uint32_t scl;                   /* SCL IO Pin */
    uint32_t sda;                   /* SDA IO Pin */
    uint32_t ticks_per_cycle;       /* Clock delay time */
    volatile I2C_MAILBOX *mailbox;  /* pointer to a mailbox */
} I2C_INIT;

#define I2C_READ        1
#define I2C_WRITE       0

#endif

/*
Copyright (c) 2012 David Michael Betz

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
