//
//    Test program for I2C bus driver.
//

#include <stdio.h>
#include <propeller.h>
#include "MiniIMU9.h"

void pauseSec(int secs)
{
    waitcnt((CLKFREQ*secs) + CNT);
}

void pauseMSec(int msecs)
{
    int delay = ((CLKFREQ/1000)*msecs);
    waitcnt(delay+CNT);
}

int main(void)
{
    int16_t accelValues[3];
    int16_t magValues[3];
    int16_t gyroValues[3];
    int     result = 0;
    
    pauseSec(3);
    printf("%c", 16);
    printf("<<<< I2C IMU Test Program 0.5 >>>>\n");
    
    I2C i2cBus(26,27,400000);
    result = i2cBus.GetCog();
    
    
    if (result >= 0)
        printf("  Bus operating on COG: %d   Address: %x\n\n", result, &i2cBus);
    else
    {
        printf("  Bus failed to start\n\n");
        return -1;
    }
    

    pauseSec(3);
    printf("\n\nCreating IMU\n", result);
    MiniIMU9 imuDevice;
    imuDevice.Init(i2cBus);
    result = imuDevice.IsOnline();
    printf("IMU Online: %d\n\n", result);

    imuDevice.m_accel.SetDataRate(1);
    imuDevice.m_accel.SetFullScale(0);
    imuDevice.m_compass.SetGain(4);
    imuDevice.m_compass.SetDataRate(6);
    
    printf("Accelerometer Config:\n");
    printf("  Config Reg1 : %x\n", imuDevice.m_accel.ReadReg(A_CTRL_REG1));
    printf("  Config Reg2 : %x\n", imuDevice.m_accel.ReadReg(A_CTRL_REG2));
    printf("  Config Reg3 : %x\n", imuDevice.m_accel.ReadReg(A_CTRL_REG3));
    printf("  Config Reg4 : %x\n", imuDevice.m_accel.ReadReg(A_CTRL_REG4));
    imuDevice.m_accel.ReadAllChannels(accelValues);
    printf("  Block Read   : X %d   Y %d   Z %d\n", accelValues[0], accelValues[1], accelValues[2]);
    pauseSec(2);
    
    
    printf("\n\nCompass Config:\n");
    printf("  Config RegA  : %x\n", imuDevice.m_compass.ReadReg(M_CRA_REG));
    printf("  Config RegB  : %x\n", imuDevice.m_compass.ReadReg(M_CRB_REG));
    printf("  Config RegMR : %x\n", imuDevice.m_compass.ReadReg(M_MR_REG));
    imuDevice.m_compass.ReadAllChannels(magValues);
    printf("  Block Read   : X %d   Y %d   Z %d\n", magValues[0], magValues[1], magValues[2]);
    pauseSec(2);
    

    printf("\n\nGyro Config:\n");
    printf("  Config Reg1  : %x\n", imuDevice.m_gyro.ReadReg(G_CTRL_REG1));
    printf("  Config Reg2  : %x\n", imuDevice.m_gyro.ReadReg(G_CTRL_REG2));
    printf("  Config Reg3  : %x\n", imuDevice.m_gyro.ReadReg(G_CTRL_REG3));
    printf("  Config Reg4  : %x\n", imuDevice.m_gyro.ReadReg(G_CTRL_REG4));
    printf("  Config Reg5  : %x\n", imuDevice.m_gyro.ReadReg(G_CTRL_REG5));
    imuDevice.m_gyro.ReadAllChannels(magValues);
    printf("  Block Read   : X %d   Y %d   Z %d\n", gyroValues[0], gyroValues[1], gyroValues[2]);
    pauseSec(2);

    vector a, m, g;
    while(1)
    {
        imuDevice.m_accel.ReadAllChannels(accelValues);
        imuDevice.m_compass.ReadAllChannels(magValues);
        imuDevice.m_gyro.ReadAllChannels(gyroValues);
        a = imuDevice.GetAccelVector();
        m = imuDevice.GetMagVector();

        printf("%c", 16);
        printf("\nA: {%d}  {%d}  {%d}   ", a.x, a.y, a.z);
        printf("M: <%d>  <%d>  <%d>   ", m.x, m.y, m.z);
        printf("G: [%d]  [%d]  [%d]\n", gyroValues[0], gyroValues[1], gyroValues[2]);
        printf("Heading: %d", 0);

        pauseMSec(200);
    }



return 0;    
}

