#include "MiniIMU9.h"
#include <stdio.h>

MiniIMU9::MiniIMU9()
{
	m_ready = 0;
	m_accelScale = 0;
}


MiniIMU9::~MiniIMU9()
{
	// TODO Auto-generated destructor stub
}


void MiniIMU9::Init(I2C& bus, int aUpdate, int rate)
{

	// Setup the devices on their default addresses.
	m_accel.Init(bus);
	m_compass.Init(bus);
	m_gyro.Init(bus);

	if (IsOnline())
	{
		m_accel.Enable(1);
		m_compass.Enable(1);
		m_gyro.Enable(1);

		m_accel.SetByteOrder(1);
		m_accel.SetUpdateMode(1);
		m_accel.SetFullScale(0);
		m_gyro.SetByteOrder(1);
		m_gyro.SetUpdateMode(1);

		m_ready = 1;
	}

	  m_magMax.x = +540; m_magMax.y = +500; m_magMax.z = 180;
	  m_magMin.x = -520; m_magMin.y = -570; m_magMin.z = -770;

	  m_accelScale = 1024;

	  m_accelCor.x = 0;
	  m_accelCor.y = 0;
	  m_accelCor.z = 0;

	// If requested hook the devices to the auto update core to allow for
	// buffered and filtered data.
	//if (aUpdate)
	//{
		//TODO Auto update cog start
	//}
}


int MiniIMU9::IsOnline()
{
	return (m_accel.IsOnline() && m_compass.IsOnline() && m_gyro.IsOnline());
}


void MiniIMU9::SetAccelCal(vector max, vector min)
{
	m_accelCor.x = (min.x + max.x)/2;
	m_accelCor.y = (min.y + max.y)/2;
	m_accelCor.z = (min.z + max.z)/2;
}


void MiniIMU9::Update()
{
	m_acc = GetAccelVector();
	m_mag = GetMagVector();
}


int MiniIMU9::GetPitch()
{
	int result = 0;
	int x_accel = m_accel.ReadChannel(0) - m_accelCor.x;
	int degPer = 90*10000 / m_accelScale;

	result = (degPer * x_accel)/10000;

	if (result > 90)
		result = 90;

	if (result < -90)
		result = -90;

	return result;
}


vector MiniIMU9::GetAccelVector()
{
	vector result;
	int16_t accelData[3];
	if (m_accel.ReadAllChannels(accelData) == 0)
	{
		result.x = accelData[0] /*- m_accelCor.x*/;
		result.y = accelData[1] /*- m_accelCor.y*/;
		result.z = accelData[2] /*- m_accelCor.z*/;
	}
	else
		result.x = result.y = result.z = -1.0;

	return result;
}


vector MiniIMU9::GetMagVector()
{
	vector result;
	int16_t magData[3];
	if (m_compass.ReadAllChannels(magData) == 0)
	{
		result.x = magData[0];
		result.y = magData[1];
		result.z = magData[2];
	}
	else
		result.x = result.y = result.z = -1.0;

	return result;
}
