#ifndef MINI_IMU_9_H_
#define MINI_IMU_9_H_

#include <propeller.h>
#include "I2CBus.h"
#include "MiniIMU9Accel.h"
#include "MiniIMU9Compass.h"
#include "MiniIMU9Gyro.h"

typedef struct vector
{
	int16_t x, y, z;
}vector;

class MiniIMU9
{
public:
	MiniIMU9Accel 		m_accel;
	MiniIMU9Compass 	m_compass;
	MiniIMU9Gyro		m_gyro;

private:
	int m_ready;
	int m_accelScale;	// Absolute value for 1g of Acceleration.

	vector m_acc;
	vector m_mag;
	vector m_gyr;

	vector m_magMax;
	vector m_magMin;
	vector m_magCor;

	vector m_accelMax;
	vector m_accelMin;
	vector m_accelCor;

public:
	MiniIMU9();
	~MiniIMU9();
	void Init(I2C& bus, int aUpdate=0, int rate=50);
	void SetAccelCal(vector max, vector min);
	void SetMagCal(vector max, vector min);

	int IsOnline();
	void Update();
	int GetPitch();
	int GetRoll();

	vector GetAccelVector();
	vector GetMagVector();
	vector GetGyroVector();

private:
	int 	ComputeHeading();

};


#endif

