/*
 * MiniIMU9Gyro.cpp
 *
 *  Created on: Aug 9, 2012
 *      Author: kcrane
 */

#include"MiniIMU9Gyro.h"

MiniIMU9Gyro::MiniIMU9Gyro()
{
    m_bus  = 0;
    m_addr = 0;
}


MiniIMU9Gyro::~MiniIMU9Gyro()
{
    m_bus  = 0;
    m_addr = 0;
}


void MiniIMU9Gyro::Init(I2C& bus)
{
    Init(bus, MIMU_GYRO_ADR);
}


void MiniIMU9Gyro::Init(I2C& bus, uint8_t addr)
{
    m_bus = &bus;
    m_addr = addr;

    //SetUpdateMode(1);
    //SetByteOrder(1);
}


int MiniIMU9Gyro::IsOnline()
{
    if(m_bus)
        return m_bus->DevPresent(m_addr);
    else
        return -1;
}


int MiniIMU9Gyro::WriteReg(int reg, int value)
{
    if(m_bus)
    {
        if (reg >= G_CTRL_REG1 && reg <= G_INT1_TSH_ZL_REG)
            return m_bus->TXByte(m_addr, reg, 1, value);
        else
            return -1;
    }
    else
        return -1;
}


int MiniIMU9Gyro::ReadReg(int reg)
{
    if(m_bus)
    {
        if (reg >= G_CTRL_REG1 && reg <= G_INT1_TSH_ZL_REG)
            return m_bus->RXByte(m_addr, reg, 1);
        else
            return -1;
    }
    else
        return -1;
}


int16_t MiniIMU9Gyro::ReadChannel(uint8_t chan)
{
    if(m_bus)
    {
        int16_t chanVal = 0;
        uint8_t reg     = G_OUT_X_L + (chan << 1);
        reg |= G_AUTO_INC_M;

        chanVal = m_bus->RXWord(m_addr, reg, 1);

        return chanVal;
    }
    else
        return -1;
}


int    MiniIMU9Gyro::ReadAllChannels(int16_t* buffer)
{
    if(m_bus)
    {
        uint8_t reg = G_OUT_X_L | G_AUTO_INC_M;
        int        result = -1;
        uint8_t temp[6];

        result = m_bus->RXBuf(m_addr, temp, 6, reg, 1);

        int n=0;
        for(int i=0; i<3; i++)
        {
            buffer[i] = (temp[n] << 8) | temp[n+1];
            buffer[i] >>= 4;
            n += 2;
        }

        return result;
    }
    else
        return -1;
}


int MiniIMU9Gyro::Enable(uint8_t val)
{
    if(m_bus)
    {
        int result = 0;
        uint8_t regVal = ReadReg(G_CTRL_REG1);

        if (val < 2)
        {
            val <<= 3;                             // Move to correct bit position
            if (val)
                val |= G_AXIS_EN_M;                // If turning on then enable all axes.

            regVal &= ~(G_POWERMD_M|G_AXIS_EN_M);   // Clear current bits
            regVal |= val;                        // Set new bits
            result = WriteReg(G_CTRL_REG1, regVal); // Write final value
        }
        else
          result = -1;

        return result;
    }
    else
        return -1;
}








int MiniIMU9Gyro::SetByteOrder(uint8_t val)
{
    if(m_bus)
    {
        int result = 0;
        uint8_t regVal = ReadReg(G_CTRL_REG4);

        if (val < 2)
        {
            val <<= 6;                   // Move to correct bit position
            regVal &= ~G_DATA_ORDER_M;   // Clear current bits
            regVal |= val;               // Set new bits
            result = WriteReg(G_CTRL_REG4, regVal);
        }
        else
            result = -1;

        return result;
    }
    else
        return -1;
}


int MiniIMU9Gyro::SetUpdateMode(uint8_t val)
{
    if(m_bus)
    {
        int result = 0;
        uint8_t regVal = ReadReg(G_CTRL_REG4);

        if (val < 2)
        {
            val <<= 7;                  // Move to correct bit position
            regVal &= ~G_UPDATE_MODE_M; // Clear current bits
            regVal |= val;              // Set new bits
            result = WriteReg(G_CTRL_REG4, regVal);
        }
        else
            result = -1;

        return result;
    }
    else
        return -1;
}
